@@ -1,27 +1,25 @@
-#!/usr/bin/perl
-
 package FusionInventory::Agent;
 
-use Cwd;
-use English;
-
 use strict;
 use warnings;
 
+use Cwd;
+use English qw(-no_match_vars);
+
 use File::Path;
 
-# THIS IS AN UGLY WORKAROUND FOR
-# http://rt.cpan.org/Ticket/Display.html?id=38067
 use XML::Simple;
 use Sys::Hostname;
 
-our $VERSION = '2.0.5';
+our $VERSION = '2.1.1';
 $ENV{LC_ALL} = 'C'; # Turn off localised output for commands
 $ENV{LANG} = 'C'; # Turn off localised output for commands
 
+# THIS IS AN UGLY WORKAROUND FOR
+# http://rt.cpan.org/Ticket/Display.html?id=38067
 eval {XMLout("<a>b</a>");};
-if ($@){
-    no strict 'refs';
+if ($EVAL_ERROR) {
+    no strict 'refs'; ## no critic
     ${*{"XML::SAX::"}{HASH}{'parsers'}} = sub {
         return [ {
             'Features' => {
@@ -45,54 +43,87 @@ use FusionInventory::Agent::Task;
 use FusionInventory::Agent::AccountInfo;
 use FusionInventory::Agent::Storage;
 use FusionInventory::Agent::Config;
-use FusionInventory::Agent::RPC;
 use FusionInventory::Agent::Targets;
 
 sub new {
-    my (undef, $self) = @_;
+    my ($class, $params) = @_;
 
-############################
-#### CLI parameters ########
-############################
+    my $self = {};
     my $config = $self->{config} = FusionInventory::Agent::Config::load();
 
+    if ($params->{winService}) {
+        $config->{winService} = 1;
+    }
+
     # TODO: should be in Config.pm
     if ($config->{logfile}) {
-        $config->{logger} = 'File';
+        $config->{logger} .= ',File';
     }
 
-    my $logger = $self->{logger} = new FusionInventory::Logger ({
-            config => $config
-        });
+    my $logger = $self->{logger} = FusionInventory::Logger->new({
+        config => $config
+    });
 
-# $< == $REAL_USER_ID
-    if ( $< ne '0' ) {
+    if ( $REAL_USER_ID != 0 ) {
         $logger->info("You should run this program as super-user.");
     }
 
-    if (!-d $config->{basevardir} && !mkpath($config->{basevardir})) {
-        $logger->error("Failed to create ".$config->{basevardir});
+    if (!-d $config->{basevardir} && !mkpath($config->{basevardir}, {error => undef})) {
+        $logger->error(
+            "Failed to create ".$config->{basevardir}.
+            " Please use --basevardir to point to a R/W directory."
+        );
     }
 
-    if (not $config->{scanhomedirs}) {
+    if (not $config->{'scan-homedirs'}) {
         $logger->debug("--scan-homedirs missing. Don't scan user directories");
     }
 
-    if ($config->{nosoft}) {
-        $logger->info("the parameter --nosoft is deprecated and may be removed in a future release, please use --nosoftware instead.");
-        $config->{nosoftware} = 1
+    if ($config->{nosoft} || $config->{nosoftware}) {
+        $logger->info("the parameter --nosoft and --nosoftware are ".
+            "deprecated and may be removed in a future release, ".
+            "please use --no-software instead.");
+        $config->{'no-software'} = 1
     }
 
+    if (!-d $config->{'share-dir'}) {
+        $logger->error("share-dir doesn't existe ".
+            "(".$config->{'share-dir'}.")");
+    }
 
-    my $hostname = hostname; # Sys::Hostname
+    #my $hostname = Encode::from_to(hostname(), "cp1251", "UTF-8");
+    my $hostname;
+  
 
-# /!\ $rootStorage save/read data in 'basevardir', not in a target directory!
-    my $rootStorage = new FusionInventory::Agent::Storage({
+    if ($OSNAME =~ /^MSWin/) {
+        eval '
+use Encode;
+use Win32::API;
+
+	my $GetComputerName = new Win32::API("kernel32", "GetComputerNameExW", ["I", "P", "P"],
+"N");
+my $lpBuffer = "\x00" x 1024;
+my $N=1024;#pack ("c4", 160,0,0,0);
+
+my $return = $GetComputerName->Call(3, $lpBuffer,$N);
+
+# GetComputerNameExW returns the string in UTF16, we have to change it
+# to UTF8
+$hostname = encode("UTF-8", substr(decode("UCS-2le", $lpBuffer),0,ord $N));';
+    } else {
+        $hostname = hostname();
+    }
+
+    # $rootStorage save/read data in 'basevardir', not in a target directory!
+    my $rootStorage = FusionInventory::Agent::Storage->new({
         config => $config
     });
     my $myRootData = $rootStorage->restore();
 
-    if (!defined($myRootData->{previousHostname}) || defined($myRootData->{previousHostname}) &&  ($myRootData->{previousHostname} ne $hostname)) {
+    if (
+        !defined($myRootData->{previousHostname}) ||
+        $myRootData->{previousHostname} ne $hostname
+    ) {
         my ($YEAR, $MONTH , $DAY, $HOUR, $MIN, $SEC) = (localtime
             (time))[5,4,3,2,1,0];
         $self->{deviceid} =sprintf "%s-%02d-%02d-%02d-%02d-%02d-%02d",
@@ -105,37 +136,32 @@ sub new {
         $self->{deviceid} = $myRootData->{deviceid}
     }
 
-
-############################
-#### Objects initilisation
-############################
-
-
-######
-    $self->{targets} = new FusionInventory::Agent::Targets({
-
-            logger => $logger,
-            config => $config,
-            deviceid => $self->{deviceid}
-            
-        });
+    $self->{targets} = FusionInventory::Agent::Targets->new({
+        logger => $logger,
+        config => $config,
+        deviceid => $self->{deviceid}
+    });
     my $targets = $self->{targets};
 
+    if (!$targets->numberOfTargets()) {
+        $logger->error("No target defined. Please use ".
+            "--server=SERVER or --local=/directory");
+        exit 1;
+    }
+
     if ($config->{daemon}) {
 
         $logger->debug("Time to call Proc::Daemon");
 
         my $cwd = getcwd();
         eval { require Proc::Daemon; };
-        if ($@) {
+        if ($EVAL_ERROR) {
             print "Can't load Proc::Daemon. Is the module installed?";
             exit 1;
         }
         Proc::Daemon::Init();
         $logger->debug("Daemon started");
-        if (isAgentAlreadyRunning({
-                    logger => $logger,
-                })) {
+        if (isAgentAlreadyRunning({ logger => $logger })) {
             $logger->debug("An agent is already runnnig, exiting...");
             exit 1;
         }
@@ -144,18 +170,24 @@ sub new {
         chdir $cwd if $config->{devlib};
 
     }
-    $self->{rpc} = new FusionInventory::Agent::RPC ({
-          
-            logger => $logger,
-            config => $config,
-            targets => $targets,
-  
-        });
+
+    # threads and HTTP::Daemon are optional and so this module
+    # may fail to load.
+    if (eval "use FusionInventory::Agent::RPC;1;") {
+        $self->{rpc} = FusionInventory::Agent::RPC->new({
+                logger => $logger,
+                config => $config,
+                targets => $targets,
+            });
+    } else {
+        $logger->debug("Failed to load RPC module: $EVAL_ERROR");
+    }
 
     $logger->debug("FusionInventory Agent initialised");
 
-    bless $self;
+    bless $self, $class;
 
+    return $self;
 }
 
 sub isAgentAlreadyRunning {
@@ -163,7 +195,7 @@ sub isAgentAlreadyRunning {
     my $logger = $params->{logger};
     # TODO add a workaround if Proc::PID::File is not installed
     eval { require Proc::PID::File; };
-    if(!$@) {
+    if(!$EVAL_ERROR) {
         $logger->debug('Proc::PID::File avalaible, checking for pid file');
         if (Proc::PID::File->running()) {
             $logger->debug('parent process already exists');
@@ -182,16 +214,8 @@ sub main {
     my $logger = $self->{logger};
     my $targets = $self->{targets};
     my $rpc = $self->{rpc};
+    $rpc && $rpc->setCurrentStatus("waiting");
 
-
-
-#####################################
-################ MAIN ###############
-#####################################
-
-
-#######################################################
-#######################################################
     while (my $target = $targets->getNext()) {
 
         my $exitcode = 0;
@@ -200,23 +224,19 @@ sub main {
         my $prologresp;
         if ($target->{type} eq 'server') {
 
-            my $network = new FusionInventory::Agent::Network ({
+            my $network = FusionInventory::Agent::Network->new({
+                logger => $logger,
+                config => $config,
+                target => $target,
+            });
 
-                    logger => $logger,
-                    config => $config,
-                    target => $target,
-
-                });
-
-            my $prolog = new FusionInventory::Agent::XML::Query::Prolog({
-
-                    accountinfo => $target->{accountinfo}, #? XXX
-                    logger => $logger,
-                    config => $config,
-                    rpc => $rpc,
-                    target => $target
-
-                });
+            my $prolog = FusionInventory::Agent::XML::Query::Prolog->new({
+                accountinfo => $target->{accountinfo}, #? XXX
+                logger => $logger,
+                config => $config,
+                rpc => $rpc,
+                target => $target
+            });
 
             # TODO Don't mix settings and temp value
             $prologresp = $network->send({message => $prolog});
@@ -231,46 +251,42 @@ sub main {
         }
 
 
-        my $storage = new FusionInventory::Agent::Storage({
-
-                config => $config,
-                logger => $logger,
-                target => $target,
-
-            });
+        my $storage = FusionInventory::Agent::Storage->new({
+            config => $config,
+            logger => $logger,
+            target => $target,
+        });
         $storage->save({
-
             data => {
                 config => $config,
                 target => $target,
                 #logger => $logger, # XXX Needed?
                 prologresp => $prologresp
             }
+        });
 
+        my @tasks = qw/
+            Inventory
+            OcsDeploy
+            WakeOnLan
+            SNMPQuery
+            NetDiscovery
+            Ping
+            /;
+
+        foreach my $module (@tasks) {
+            my $task = FusionInventory::Agent::Task->new({
+                config => $config,
+                logger => $logger,
+                module => $module,
+                target => $target,
             });
 
-
-        my %taskOptions = (
-            Inventory => 'noinventory',
-            OcsDeploy => 'noocsdeploy',
-            WakeOnLan => 'nowakeonlan',
-            SNMPQuery => 'nosnmpquery',
-            NetDiscovery => 'nonetdiscovery'
-            );
-
-        foreach my $module (keys %taskOptions) {
-            my $task = new FusionInventory::Agent::Task({
-                    config => $config,
-                    logger => $logger,
-                    module => $module,
-                    name => $taskOptions{$module},
-                    target => $target,
-
-                });
-
+            $rpc && $rpc->setCurrentStatus("running task $module");
             next unless $task;
             $task->run();
         }
+        $rpc && $rpc->setCurrentStatus("waiting");
 
         if (!$config->{debug}) {
             # In debug mode, I do not clean the FusionInventory-Agent.dump
