@@ -1,5 +1,9 @@
 package FusionInventory::Agent::Task::Inventory::OS::Linux::Archs::m68k::CPU;
+
 use strict;
+use warnings;
+
+use English qw(-no_match_vars);
 
 sub isInventoryEnabled { can_read("/proc/cpuinfo") }
 
@@ -7,13 +11,18 @@ sub doInventory {
     my $params = shift;
     my $inventory = $params->{inventory};
 
+    my $handle;
+    if (!open $handle, '<', '/proc/cpuinfo') {
+        warn "Can't open /proc/cpuinfo: $ERRNO";
+        return;
+    }
+
     my @cpu;
     my $current;
-    open CPUINFO, "</proc/cpuinfo" or warn;
-    foreach(<CPUINFO>) {
+
+    while (<$handle>) {
         print;
         if (/^CPU\s+:\s*:/) {
-
             if ($current) {
                 $inventory->addCPU($current);
             }
@@ -21,17 +30,15 @@ sub doInventory {
             $current = {
                 ARCH => 'm68k',
             };
-
         } else {
-
             $current->{TYPE} = $1 if /CPU:\s+(\S.*)/;
             $current->{SPEED} = $1 if /Clocking:\s+:\s+(\S.*)/;
-
         }
     }
+    close $handle;
 
     # The last one
     $inventory->addCPU($current);
 }
 
-1
+1;
